/*============================================================================
                      FORMAL DIFFERENTIATION
The input expression to be differentiated is given according to the following
grammar:
   element ::= (top)|'0'|...|'9'|'a'|...|'z'
   primary ::= element   | - primary                     (unary minus)
   term	   ::= primary   | primary ^ term   	         (exponentiation)
   expr	   ::= term      | term * expr    | term / expr  (times and division)
   top	   ::= expr      | expr + top     | expr - top   (plus and minus)
The binding priorities are: most binding   -      (unary minus)
                                 |         ^      (exponentiation)
                                 |         *   /  (times and division)
                            least binding  +   -  (plus and binary minus))
In the exponentiation the exponent should not be dependent on the variable
w.r.t. which we differentiate.
Type semicolon and carriage return at the end of input expression. In the
output expression > is printed as ->.
Right associativity is assumed. This is usual for times (e.g., a*b*c =
a*(b*c)), but it is not usual for minus. Thus, a-b-c = a-(b-c), not (a-b)-c.
------------------------------------------------------------------------------
No use of the 'delete' instruction is made for improving storage utilization.
This is not a big disadvantage because when exiting a recursive call all local
variables are discarded.
The output is not simplified: for instance, 0*x, y*1, and 0+1 are not
simplified to x, y, and 1, respectively.
=========================================================================== */
#include <iostream.h>
#include <stdlib.h>   // file needed for the EXIT_SUCCESS and the EXIT_FAILURE
enum binop {exp,times,over,plus,minus};
enum Tag {variabletree, constanttree, unarymintree, binarytree};
struct tree {
       struct Variable {char vname;};
       struct Constant {char cname;};
       struct Unary    {tree* son;};
       struct Binary   {binop bop; tree* left; tree* right;};
       union {Variable v; Constant c; Unary u; Binary b;};
       Tag tag;};
typedef tree* treeptr;
//----------------------------------------------------------------------------
char x; char y;
//----------------------------------------------------------------------------
//                    constanttree from character z
treeptr t_from_ch(char z)
{treeptr t=new tree; t->tag=constanttree; t->c.cname=z; return t;}
//----------------------------------------------------------------------------
//                    printing a tree
void printtree(treeptr t)
{if (t==NULL) {cout << "Nil_tree";}
else {switch (t->tag){
case variabletree: cout << t->v.vname; break;
case constanttree: cout << t->c.cname; break;
case unarymintree: cout << "(-"; printtree(t->u.son); cout << ")"; break;
case binarytree:   cout << "("; printtree(t->b.left);
        switch (t->b.bop){
          case exp:   cout << "^"; break;
          case times: cout << "*"; break;
          case over:  cout << "/"; break;
          case plus:  cout << "+"; break;
          case minus: cout << "-"; break;
        default: cout << "Error 1 in printtree!";};
        printtree(t->b.right); cout << ")"; break;
default: cout << "Error 2 in printtree!";
         exit(EXIT_FAILURE);}};}
//----------------------------------------------------------------------------
//                    declarations for mutual recursion
void parsetop    (treeptr &t);
void parseexpr   (treeptr &t);
void parseterm   (treeptr &t);
void parseprimary(treeptr &t);
void parseelem   (treeptr &t);
//----------------------------------------------------------------------------
//                    parsing the input expression
void parsetop(treeptr &t)
{treeptr ntpD1=new tree; parseexpr(ntpD1);
if (x=='+') {treeptr ntpD=new tree; ntpD->tag=binarytree;
             ntpD->b.left=ntpD1; ntpD->b.bop=plus;
             cin >> x; parsetop(ntpD->b.right); t=ntpD;}
else if (x=='-') {treeptr ntpD=new tree; ntpD->tag=binarytree;
             ntpD->b.left=ntpD1; ntpD->b.bop=minus;
             cin >> x; parsetop(ntpD->b.right); t=ntpD;}
else t=ntpD1;}
//-------
void parseexpr(treeptr &t)
{treeptr ntpE1=new tree; parseterm(ntpE1);
if (x=='*') {treeptr ntpE=new tree; ntpE->tag=binarytree;
             ntpE->b.left=ntpE1; ntpE->b.bop=times;
             cin >> x; parseexpr(ntpE->b.right); t=ntpE;}
else if (x=='/') {treeptr ntpE=new tree; ntpE->tag=binarytree;
             ntpE->b.left=ntpE1; ntpE->b.bop=over;
             cin >> x; parseexpr(ntpE->b.right); t=ntpE;}
else t=ntpE1;}
//-------
void parseterm(treeptr &t)
{treeptr ntpT1=new tree;
parseprimary(ntpT1);
if (x=='^') {treeptr ntpT=new tree; ntpT->tag=binarytree;
             ntpT->b.left=ntpT1; ntpT->b.bop=exp;
             cin >> x; parseterm(ntpT->b.right); t=ntpT;}
else t=ntpT1;}
//-------
void parseprimary(treeptr &t)
{treeptr ntpP=new tree;
if (x =='-') {ntpP->tag=unarymintree; cin >> x;
              treeptr ntpP1=new tree; parseprimary(ntpP1);
              ntpP->u.son=ntpP1;}
else {parseelem(ntpP);};
t=ntpP;}
//-------
void parseelem(treeptr &t)
{treeptr ntpV=new tree;
if (('a' <= x) && (x <= 'z'))
     {ntpV->tag=variabletree; ntpV->v.vname=x; cin >> x;}
else if (('0' <= x) && (x <= '9'))
     {ntpV->tag=constanttree; ntpV->c.cname=x; cin >> x;}
else {if (x=='(') {cin >> x; parsetop(ntpV); if (x==')') {cin >> x;};};};
t=ntpV;}
//----------------------------------------------------------------------------
treeptr differentiate(treeptr t)
{if (t==NULL) {cout << "Error 1: Derivative of Nil_tree!\n";} else
{treeptr upptr=new tree;    treeptr exp_ptr=new tree;
 treeptr cexp_ptr=new tree;
 treeptr ptr_left=new tree; treeptr ptr_right=new tree;
 treeptr prt_numl=new tree; treeptr prt_numr=new tree;
 treeptr numptr=new tree;   treeptr denptr=new tree;
 treeptr newt=new tree;
// needed duplication of t into newt because pointer and recursion do not work
// always well (as in Pascal)!
switch (t->tag){
case variabletree: if (t->v.vname==y) {newt->tag=constanttree;
                                       newt->c.cname='1';}
                   else {newt->tag=constanttree; newt->c.cname='0';};
                   return newt; break;
case constanttree: newt->tag=constanttree; newt->c.cname='0';
                   return newt; break;
case unarymintree: newt->tag=unarymintree;
                   newt->u.son=differentiate(t->u.son); return newt; break;
case binarytree:   newt->tag=binarytree;
        switch (t->b.bop){
        case exp   : newt->b.bop=times;
                     //-----------
                     upptr->tag=binarytree; upptr->b.bop=minus;
                     upptr->b.left=t->b.right; upptr->b.right=t_from_ch('1');
                     //-----------
                     exp_ptr->tag=binarytree; exp_ptr->b.bop=exp;
                     exp_ptr->b.left=t->b.left; exp_ptr->b.right=upptr;
                     //-----------
                     cexp_ptr->tag=binarytree; cexp_ptr->b.bop=times;
                     cexp_ptr->b.left=t->b.right; cexp_ptr->b.right=exp_ptr;
                     //-----------
                     newt->b.left=cexp_ptr;
                     newt->b.right=differentiate(t->b.left);
                     return newt; break;
        case times : newt->b.bop=plus;
                     //-----------
                     ptr_left->tag=binarytree; ptr_left->b.bop=times;
                     ptr_left->b.left=differentiate(t->b.left);
                     ptr_left->b.right=t->b.right;
                     //-----------
                     ptr_right->tag=binarytree; ptr_right->b.bop=times;
                     ptr_right->b.left=t->b.left;
                     ptr_right->b.right=differentiate(t->b.right);
                     //-----------
                     newt->b.left=ptr_left; newt->b.right=ptr_right;
                     return newt; break;
        case over  : newt->b.bop=over;
                     //-----------
                     prt_numl->tag=binarytree; prt_numl->b.bop=times;
                     prt_numl->b.left=differentiate(t->b.left);
                     prt_numl->b.right=t->b.right;
                     //-----------
                     prt_numr->tag=binarytree; prt_numr->b.bop=times;
                     prt_numr->b.left=t->b.left;
                     prt_numr->b.right=differentiate(t->b.right);
                     //-----------
                     numptr->tag=binarytree; numptr->b.bop=minus;
                     numptr->b.left=prt_numl; numptr->b.right=prt_numr;
                     //-----------
                     denptr->tag=binarytree; denptr->b.bop=exp;
                     denptr->b.left=t->b.right; denptr->b.right=t_from_ch('2');
                     //-----------
                     newt->b.left=numptr; newt->b.right=denptr;
                     return newt; break;
        case plus  : newt->b.bop=plus;
                     newt->b.left=differentiate(t->b.left);
                     newt->b.right=differentiate(t->b.right);
                     return newt; break;
        case minus : newt->b.bop=minus;
                     newt->b.left=differentiate(t->b.left);
                     newt->b.right=differentiate(t->b.right);
                     return newt; break;
        default: cout << "Error 2 in differentiate!\n";
             exit(EXIT_FAILURE);
        }; // end binary_ switch
                   break;
default: cout << "Error 3 in differentiate!\n"; exit(EXIT_FAILURE);
}; // end tag
}; // end switch
}  // end differentiate
//----------------------------------------------------------------------------
void main()
{cout << "\nWrite an expression with variables from 'a' to 'z', please.\n";
cout << "You may use the constants: 0,...,9.\n";
cout << "You may use the operators - (unary or binary), +, *, /, ^,\n";
cout << "and the parentheses (,). Then type ; and CR.\n";
treeptr ntp=new tree;
cin >> x; parsetop(ntp);
cout << "\nExpression E to be differentiated:\n"; printtree(ntp);
cout << "\nDerivative w.r.t.? Which of [a..z]? Then type CR.\n";
cin >> y; ntp=differentiate(ntp);
cout << "\ndE/d"<<y<<":\n"; printtree(ntp); cout << "\n\n";
}
/* input:  output:
------------------------------------------------------------------------------
           Write an expression with variables from 'a' to 'z', please.
           You may use the constants: 0,...,9.
           You may use the operators - (unary or binary), +, *, /, ^,
           and the parentheses (,). Then type ; and CR.
x*(y/x);CR
           Expression E to be differentiated:
           (x*(y/x))
           Derivative w.r.t.? Which of [a..z]? Then type CR.
xCR
           dE/dx:
           ((1*(y/x))+(x*(((0*x)-(y*1))/(x^2))))
--------------------------------------------------------------------------- */